using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.SqlServer.Management.Smo;
using Microsoft.SqlServer.Management.Common;

using DomainObjects.Facade.Command;


namespace com.diagast.testdbmapping {

    public class ObjectFactory {
        private static ObjectFactory factory = new ObjectFactory();
        private DomainObjectsDbAccess dbaccess;

        private ObjectFactory() {
            this.createdbschema();
            dbaccess = DomainObjectsDbAccess.Instance;
        }

        public static ObjectFactory getFactory() {
            return factory;
        }

        public void testDemandeMappingWithQueryOnly() {

            //test query
            Criteria criteria = new Criteria();

            // The line below tell DomainObjects: "Query for an instance of type Planet
            // where the field '_name' has a value of 'Earth'"
            criteria.AddEqualTo(Types.Demande.Field._sampleid, 36);

            List<Demande> list = QueryFacade.FindObjectSet<Demande>(criteria);
            foreach (Demande dem in list) {
                dem.Sampleid = 48;
                dem.Patientid = "47";
                dbaccess.UpdateObjectInDB(dem);
            }

        }

        public void testDeleteCollectionObject() {

            List<Demande> list = dbaccess.FindAllDBObjects<Demande>();
            foreach (Demande dem in list) {
                dbaccess.DeleteObjectInDB(dem);
            }
        }

        public Demande getDemandeForSample(int sampleId) {
            //test query
            Criteria criteria = new Criteria();

            // The line below tell DomainObjects: "Query for an instance of type Planet
            // where the field '_name' has a value of 'Earth'"
            criteria.AddEqualTo(Types.Demande.Field._sampleid, sampleId);

            List<Demande> list = QueryFacade.FindObjectSet<Demande>(criteria);
            Demande dem = null;
            if (list.Count > 0) {
                dem = list[0];
            }
            return dem;
        }

        public List<Demande> getDemandesForPatient(String patientId) {
            //test query
            Criteria criteria = new Criteria();

            // The line below tell DomainObjects: "Query for an instance of type Planet
            // where the field '_name' has a value of 'Earth'"
            criteria.AddEqualTo(Types.Demande.Field._patientid, patientId);

            List<Demande> list = QueryFacade.FindObjectSet<Demande>(criteria);
            return list;
        }


        public void testQuery() {
            Demande dema = this.getDemandeForSample(1);
            Console.Out.WriteLine("Found demande for sample :" + dema);

            List<Demande> list = this.getDemandesForPatient("name_pat1" + "first_pat1");
            foreach (Demande dem in list) {
                Console.Out.WriteLine("Found demande :" + dem.PrimaryKey[0] + " patient :" + dem.Patientid);
                Sample sample = dem.Sample;
                if (sample != null) {
                    IList<Test> tlist = sample.AllTestInSample;
                    foreach (Test test in tlist) {
                        // (ICI) TWO TEST1 IN SAMPLE COLLECTION !!!!!
                        Console.Out.WriteLine("Found test type:" + test.Type+" test id :"+test.PrimaryKey[0]);
                    }

                }
            }



        }
        public void createDBObject() {
            Patient pat1 = (Patient)dbaccess.FindObjectInDBWithPrimaryKey<Patient>("name_pat1" + "first_pat1");
            pat1 = new Patient("name_pat1", "first_pat1", new DateTime());
            pat1 = (Patient)dbaccess.CreateObjectInDB<Patient>(pat1);
            Patient pat2 = new Patient("name_pat2", "first_pat2", new DateTime());
            pat2 = (Patient)dbaccess.CreateObjectInDB<Patient>(pat2);
            
            //get patient 1
            pat1 = (Patient)dbaccess.FindObjectInDBWithPrimaryKey<Patient>("name_pat1" + "first_pat1");

            //create a demande
            Demande newDemande = new Demande();
            Object obj = pat1.PrimaryKey[0];
            newDemande.Patientid = (String) pat1.PrimaryKey[0];
            newDemande = (Demande) dbaccess.CreateObjectInDB<Demande>(newDemande);

            //create sample 1
            Sample sample1 = new Sample(newDemande, "11111");
            sample1 = (Sample)dbaccess.CreateObjectInDB<Sample>(sample1);

            newDemande.Sample = sample1;
            dbaccess.UpdateObjectInDB(newDemande);

            Test test1 = new Test(sample1, "type2");
            test1 = (Test)dbaccess.CreateObjectInDB<Test>(test1);

            //ADD ONE TIME TEST1 (ICI)
            sample1.AddToAllTestInSample(test1);
            dbaccess.UpdateObjectInDB(sample1);
 
  /*          newDemande = new Demande(null);
            newDemande = (Demande)dbaccess.CreateObjectInDB<Demande>(newDemande);
            newDemande = new Demande(null);
            newDemande = (Demande)dbaccess.CreateObjectInDB<Demande>(newDemande);
            newDemande = new Demande(null);
            newDemande = (Demande)dbaccess.CreateObjectInDB<Demande>(newDemande);

            newDemande = (Demande) dbaccess.FindObjectInDBWithPrimaryKey<Demande>(newDemande.PrimaryKey);
            newDemande.Patientid = 23;
            newDemande.Sampleid = 24;
            dbaccess.UpdateObjectInDB(newDemande);

            List<Demande> list = dbaccess.FindAllDBObjects<Demande>();
            foreach (Demande dem in list) {
                dem.Sampleid = 36;
                dem.Patientid = 35;
                dbaccess.UpdateObjectInDB(dem);
            }  */
        }

        private void createdbschema() {
            try {
                //test if the db exist. if not create it and create table.
                //avoid to do the Dispose on the connection.
                using (System.Data.SqlClient.SqlConnection sqlConnection =
                  new System.Data.SqlClient.SqlConnection(@"Integrated Security=SSPI; Data Source=(local)\SQLEXPRESS"))    {

                //build a "serverConnection" with the information of the "sqlConnection"
                Microsoft.SqlServer.Management.Common.ServerConnection serverConnection =
                  new Microsoft.SqlServer.Management.Common.ServerConnection(sqlConnection);

                Server monServeur = new Server(serverConnection);

                try {
                    DatabaseCollection dbcol = monServeur.Databases;
                    //for test drop table if exist.
                    if (dbcol.Contains("TESTDBMAPPING")) {
                        Database db = dbcol["TESTDBMAPPING"];
                        db.Drop();
                    }

                    if (!dbcol.Contains("TESTDBMAPPING")) {
                        //create db
                        Database maBase = new Database(monServeur, "TESTDBMAPPING");
                        maBase.Create();

                        //create db schema
                   //     Schema sch = new Schema(maBase, "TESTDBSCHEMA");
                   //     sch.Owner = "dbo";
                   //     sch.Create();


                        //create db table
                     //   Database db = server.Databases["TESTDBMAPPING"];

                        // Create new table, called Demande
                        // object fields
                        //        private string pid;
                        //        private string patientId;
                        //        private Sample psample;

                        Table demandeTable = new Table(maBase, "DEMANDE");
                        // Add "ID" Column, which will be PK
                        Column idColumn = new Column(demandeTable, "ID");
                        idColumn.DataType = DataType.Int;
                        idColumn.Nullable = false;
                        idColumn.Identity = true;
                        idColumn.IdentitySeed = 1;
                        idColumn.IdentityIncrement = 1;
                     
                        // Add "PATIENTID" Column
                        Column patientColumn = new Column(demandeTable, "PATIENTID");
                        patientColumn.DataType = DataType.VarChar(100);
                        patientColumn.Nullable = true;
                        Column sampleColumn = new Column(demandeTable, "SAMPLEID");
                        sampleColumn.DataType = DataType.Int;
                        sampleColumn.Nullable = true;
                        // Add Columns to Table Object
                        demandeTable.Columns.Add(idColumn);
                        demandeTable.Columns.Add(patientColumn);
                        demandeTable.Columns.Add(sampleColumn);
                        // Create a PK Index for the table
                        Index index = new Index(demandeTable, "PK_DEMANDEIDINDEX");
                        index.IndexKeyType = IndexKeyType.DriPrimaryKey;
                        // The PK index will consist of 1 column, "ID"
                        index.IndexedColumns.Add(new IndexedColumn(index, "ID"));
                        // Add the new index to the table.
                        demandeTable.Indexes.Add(index);
                        // Physically create the table in the database
                     

                        demandeTable.Create();


                        // Create new table, called SAMPLE
                        // object fields
                        //        private string pid;
                        //        private String pbarcode;
                        //        private ICollection<Test> testList;

                        Table sampleTable = new Table(maBase, "SAMPLE");
                        // Add "ID" Column, which will be PK
                        idColumn = new Column(sampleTable, "ID");
                        idColumn.DataType = DataType.Int;
                        idColumn.Nullable = false;
                        idColumn.Identity = true;
                        idColumn.IdentitySeed = 1;
                        idColumn.IdentityIncrement = 1;

                        // Add "SAMPLEID" Column
                        Column iddemandeColumn = new Column(sampleTable, "DEMANDEID");
                        iddemandeColumn.DataType = DataType.Int;
                        iddemandeColumn.Nullable = false;
                        Column barcodeColumn = new Column(sampleTable, "BARCODE");
                        barcodeColumn.DataType = DataType.VarChar(50);
                        barcodeColumn.Nullable = false;
                        // Add Columns to Table Object
                        sampleTable.Columns.Add(idColumn);
                        sampleTable.Columns.Add(iddemandeColumn);
                        sampleTable.Columns.Add(barcodeColumn);
                        // Create a PK Index for the table
                        index = new Index(sampleTable, "PK_SAMPLEIDINDEX");
                        index.IndexKeyType = IndexKeyType.DriPrimaryKey;
                        // The PK index will consist of 1 column, "ID"
                        index.IndexedColumns.Add(new IndexedColumn(index, "ID"));
                        // Add the new index to the table.
                        sampleTable.Indexes.Add(index);
                        // Physically create the table in the database
                        sampleTable.Create();

                        ForeignKey fk = new ForeignKey(sampleTable, "FK_SAMPLE_DEMANDE");
                        ForeignKeyColumn fkc = new ForeignKeyColumn(fk, "DEMANDEID", "ID");
                        fk.Columns.Add(fkc);
                        fk.ReferencedTable = "DEMANDE";
                  //      fk.ReferencedTableSchema = "TESTDBSCHEMA";
                        fk.Create();

                        // Create new table, called TEST
                        // object fields
                        //        private string pid;
                        //        private string ptype;

                        Table testTable = new Table(maBase, "TEST");
                        // Add "ID" Column, which will be PK
                        idColumn = new Column(testTable, "ID");
                        idColumn.DataType = DataType.Int;
                        idColumn.Nullable = false;
                        idColumn.Identity = true;
                        idColumn.IdentitySeed = 1;
                        idColumn.IdentityIncrement = 1;

                        // Add "SAMPLEID" Column
                        Column idsampleColumn = new Column(testTable, "SAMPLEID");
                        idsampleColumn.DataType = DataType.Int;
                        idsampleColumn.Nullable = false;
                        Column typeColumn = new Column(testTable, "TYPE");
                        typeColumn.DataType = DataType.VarChar(5);
                        typeColumn.Nullable = false;
                        // Add Columns to Table Object
                        testTable.Columns.Add(idColumn);
                        testTable.Columns.Add(idsampleColumn);
                        testTable.Columns.Add(typeColumn);
                        // Create a PK Index for the table
                        index = new Index(testTable, "PK_TESTIDINDEX");
                        index.IndexKeyType = IndexKeyType.DriPrimaryKey;
                        // The PK index will consist of 1 column, "ID"
                        index.IndexedColumns.Add(new IndexedColumn(index, "ID"));
                        // Add the new index to the table.
                        testTable.Indexes.Add(index);
                        // Physically create the table in the database
                        testTable.Create();


                        fk = new ForeignKey(testTable, "FK_TEST_SAMPLE");
                        fkc = new ForeignKeyColumn(fk, "SAMPLEID", "ID");
                        fk.Columns.Add(fkc);
                        fk.ReferencedTable = "SAMPLE";
                    //    fk.ReferencedTableSchema = "TESTDBSCHEMA";
                        fk.Create();

                        // Create new table, called PATIENT
                        // object fields
                        //        private string pid;
                        //        private string pname;
                        //        private string pfirstname;
                        //        private DateTime pbirthdate;

                        Table patientTable = new Table(maBase, "PATIENT");
                        // Add "ID" Column, which will be PK
                        idColumn = new Column(patientTable, "ID");
                        idColumn.DataType = DataType.VarChar(100);
                        idColumn.Nullable = false;
                        
                        // Add "name" Column
                        Column nameColumn = new Column(patientTable, "NAME");
                        nameColumn.DataType = DataType.VarChar(60);
                        nameColumn.Nullable = false;
                        // Add "firstname" Column
                        Column firstnameColumn = new Column(patientTable, "FIRSTNAME");
                        firstnameColumn.DataType = DataType.VarChar(60);
                        firstnameColumn.Nullable = false;
                        // Add "birthdate" Column
                        Column birthColumn = new Column(patientTable, "BIRTHDATE");
                        birthColumn.DataType = DataType.DateTime;
                        birthColumn.Nullable = false;
                        // Add Columns to Table Object
                        patientTable.Columns.Add(idColumn);
                        patientTable.Columns.Add(nameColumn);
                        patientTable.Columns.Add(firstnameColumn);
                        patientTable.Columns.Add(birthColumn);
                        // Create a PK Index for the table
                        index = new Index(patientTable, "PK_PATIENTIDINDEX");
                        index.IndexKeyType = IndexKeyType.DriPrimaryKey;
                        // The PK index will consist of 1 column, "ID"
                        index.IndexedColumns.Add(new IndexedColumn(index, "ID"));
                        // Add the new index to the table.
                        patientTable.Indexes.Add(index);
                        // Physically create the table in the database
                        patientTable.Create();

                    }

                }finally    {
                    monServeur.ConnectionContext.Disconnect();
                }
              }
            } catch (Exception ex) {
                Console.Out.WriteLine(ex.StackTrace);
                throw;
            }

        }
    }
}
