/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 * 
 */

package org.openmobileis.bundle.coldupdate.server;

import java.io.File;
import java.io.FileReader;
import java.io.Reader;
import java.util.Properties;

import org.openmobileis.common.user.UserManager;
import org.openmobileis.common.user.impl.MonoUserUserManagerFactory;
import org.openmobileis.common.user.profile.ProfileDataManager;
import org.openmobileis.common.user.profile.impl.TerminalXmlFileProfilDataFactory;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.module.profiles.server.ProfilManagerSyncTarget;
import org.openmobileis.module.server.ModuleServerManager;
import org.openmobileis.services.servlet.OpenMISInit;
import org.openmobileis.synchro.algo.syncnumber.SyncNumberManager;
import org.openmobileis.synchro.algo.syncnumber.impl.EmptySyncNumberManagerDelegate;
import org.openmobileis.synchro.openmsp.server.synctarget.SynchroTargerManager;
import org.openmobileis.synchro.openmsp.server.synctarget.proxy.GenericProxySynchroTarget;

public class ColdUpdateServerOpenMISInit implements OpenMISInit {

  public ColdUpdateServerOpenMISInit() {
    super();
  }

  public void preLoadingInit() throws ServiceException {
    
    //init of log management. If the directory log exist, the logmanager log in the log/openmislog.txt file.
    // otherwise it log on the console
    Properties  props = new Properties();
    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/openmislog.txt");
    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
    if (file.exists())   FileOpenCloseLogManager.registerLogManager(props); //log to a file
    else LogManager.registerLogManager(null); //log to a file
    
    // init user manager with the default impl with one user. Login:test Pwd:test
    //to authenticate to another source, use your UserManagerFactory
    UserManager.getManager().registerUserManagerFactoryForGroup("default", new MonoUserUserManagerFactory());
    //the simpliest syncnumber. Start at 0 at each server start.
    SyncNumberManager.getManager().registerDelegate(new EmptySyncNumberManagerDelegate());
 
    
    //init profil management 
    //register a profil for a group.
    //reader contain the profil file.
    try	{
	    Reader reader = new FileReader(System.getProperty("user.dir")+"/profils/baseprofil.xml");
	    TerminalXmlFileProfilDataFactory serverprofilmanager = new TerminalXmlFileProfilDataFactory(reader);
	    ProfileDataManager.getManager().registerProfilDataFactoryForGroup("default", serverprofilmanager);
    } catch (Throwable ex)	{
    	LogManager.traceError(0, ex);
    }

    //register profil listener target.
    ProfilManagerSyncTarget target = new ProfilManagerSyncTarget();
    SynchroTargerManager.getManager().addSynchroTargetListener(target, null, null);
    
    //init module jar files synchronisation
    ModuleServerManager.getManager();
    
    // to be added to work with a servlet server.
    SynchroTargerManager.getManager().registerProxyTargetForTerminal(new GenericProxySynchroTarget(), null);      
}

  public void postLoadingInit() throws ServiceException {
  }

}
