/*
 * Created on Sun Feb 18 21:49:19 CET 2007
 */
package org.openmobileis.bundle.osgi.terminal;

import java.io.File;
import java.io.IOException;
import java.util.Properties;

import org.openmobileis.bundle.osgi.services.BunbleContextManager;
import org.openmobileis.bundle.osgi.services.TemplateRetrieverServiceImpl;
import org.openmobileis.common.context.ApplicationContextManager;
import org.openmobileis.common.context.SessionContextManager;
import org.openmobileis.common.util.PropertiesManager;
import org.openmobileis.common.util.collection.Array;
import org.openmobileis.common.util.exception.ServiceException;
import org.openmobileis.common.util.log.FileOpenCloseLogManager;
import org.openmobileis.common.util.log.LogManager;
import org.openmobileis.database.fastobjectdb.FastObjectDBManager;
import org.openmobileis.database.fastobjectdb.synchro.client.SynchroFastObjectDBManager;
import org.openmobileis.embedded.webserver.WebServer;
import org.openmobileis.embedded.webserver.templates.Freemarkerv1TemplateDelegate;
import org.openmobileis.embedded.webserver.templates.TemplateManager;
import org.openmobileis.embedded.webserver.templates.TemplateRetrieverManagerService;
import org.openmobileis.services.Service;
import org.openmobileis.services.common.ServiceManager;
import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceRegistration;

public class OpenMISBundleActivator implements BundleActivator {
	ApplicationContextManager manager = ApplicationContextManager.getManager();
	private ServiceRegistration regTemplate=null;
	private Array loadedServiceList;
	private WebServer server;

	class StartWebServertThread implements Runnable	{
		public void run()	{
	    //start web server
			try  {
				System.out.println("start WebServer");

				server = new WebServer();
	    	server.serve();
  		} catch (IOException ex)	{}
		}
	}

  /* (non-Javadoc)
   * @see org.osgi.framework.BundleActivator#start(org.osgi.framework.BundleContext)
   */
  public void start(BundleContext bundleContext) throws Exception {
	  ClassLoader bundleLoader = this.getClass().getClassLoader();
	  ClassLoader oldLoader = Thread.currentThread().getContextClassLoader();
	  Thread.currentThread().setContextClassLoader(bundleLoader);

	  BunbleContextManager.getManager().setBundleContext(bundleContext);

	  //init log
	    Properties  props = new Properties();
	    props.put("org.openmobileis.common.log.file", System.getProperty("user.dir")+"/WEB-INF/log/openmislog.txt");
	    File file = new File(System.getProperty("user.dir")+"/WEB-INF/log");
	    if (file.exists())   FileOpenCloseLogManager.registerLogManager(props); //log to a file
	    else LogManager.registerLogManager(null); //log to a file

	    //init application context and plateform
	    ApplicationContextManager.getManager();
	    ApplicationContextManager.getManager().getApplicationContext().setBundleLoaded(true);

	    //register property service.
	    PropertiesManager.getManager().addProperty("test", "testpropoerty");

	    //manage session
  	  SessionContextManager.getManager();

	    //init FODB management
	    PropertiesManager.getManager().addProperty("fastobjectdb.database.path", System.getProperty("user.dir")+"/database");
	    PropertiesManager.getManager().addProperty("fastobjectdb.database.name", "db");
	    //init synchro properties
	    PropertiesManager.getManager().addProperty("org.openmobileis.synchro.client.openmsp.numsyncpropsfile",System.getProperty("user.dir") + "/WEB-INF/conf/numsync.txt");
	    try {
	    	FastObjectDBManager.registerManager(new SynchroFastObjectDBManager()); //init synchro db to be use as default db
	    	FastObjectDBManager.getManager();
	    } catch (Throwable ex)  {
	      throw new ServiceException(ex);
	    }
	    
	    // init service management
		  ServiceManager.getManager();

	    //start seb server
	    Thread thread = new Thread(new StartWebServertThread());
	    thread.start();

	    Thread.currentThread().sleep(2000);
	    
	    //init template management
	    Freemarkerv1TemplateDelegate templateDelegate = new Freemarkerv1TemplateDelegate();
	    TemplateManager.registerTemplateDelegate(templateDelegate);
	  	String rootTemplatePath = PropertiesManager.getManager().getProperty("org.openmobileis.services.templatesDir");
	  	file = new File(rootTemplatePath);
	  	if (!file.exists())	{
	  		templateDelegate.registerTemplateRetriever(new TemplateRetrieverServiceImpl(bundleContext.getBundle().getSymbolicName(), rootTemplatePath));
	  	}
	  	regTemplate = bundleContext.registerService (TemplateRetrieverManagerService.class.getName(), templateDelegate, null);

		  loadedServiceList = ServiceManager.getManager().loadRubricLoader(new OSGIOpenMISRubricLoad());

/*	    NumSyncManagerDB.getManager();
	    ModuleManager.getManager().registerModuleManagerListener(new OSGIOpenMSPSyncListener());
	    ModuleManager.getManager().initModule(); */
		  Thread.currentThread().setContextClassLoader(oldLoader);
  }

  /* (non-Javadoc)
   * @see org.osgi.framework.BundleActivator#stop(org.osgi.framework.BundleContext)
   */
  public void stop(BundleContext context) throws Exception {
	  //unload service
	  for (int i=0; i< loadedServiceList.size(); i++)	{
	  	Service service = (Service)loadedServiceList.get(i);
	  	ServiceManager.getManager().unloadService(service.getServiceUri());
	  }
	  server.stopServer();
	  
		if (regTemplate != null) regTemplate.unregister();
 }
}