/*
 * OpenMobileIS - a free Java(TM) Framework for mobile applications Java(TM)
 * Copyright (C) 2004-2006 Philippe Delrieu
 * All rights reserved.
 * Contact: pdelrieu@openmobileis.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 *  Author : Philippe Delrieu
 *
 */
package org.openmobileis.database.fastobjectdb;

import org.openmobileis.common.util.OpenMISSerializable;

/**
 * Title:        OpenMobileIS project source <BR>
 * Description:
 * @author	Philippe Delrieu
 * @since	JDK 1.1
 * @version 1.0.
 */
public class FODBCollectionDescriptor implements OpenMISSerializable {
  static final long serialVersionUID = 5521257935120563452L;
  /**
   * Indicate that the synchronisation of the collection will be done in the two way : data change are send to the server,
   * server modification are send to the terminal.
   */
  public final static short TWO_WAY_SYNCHRO =0; //default
  /**
   * Indicate that the synchronisation of the collection will be done in the send way. Only data modified on the terminal
   * are send to the server. Server modifications are not send to the terminal.
   */
  public final static short ONE_WAY_SEND_DATA =1;
  /**
   * Indicate that the synchronisation of the collection will be done in the get way. Only data modified on the server
   * are send to the terminal. Terminal modifications are not keep and send to the server.
   */
  public final static short ONE_WAY_GET_DATA =2;
  private String collectionName;
  private Class objectType;
  private boolean synchronize;
  private boolean compressed;
  private short[] stateList; //list all collection state :
  //			0 : synchro way : define the way the synchronisation occurs.
  private String[] synchroDependList;

  /**
   *
   */
  public FODBCollectionDescriptor(String collectionName, Class objectType) {
    super();
    this.collectionName = collectionName;
    this.objectType = objectType;
    this.synchronize = true;
    this.compressed = false;
    this.stateList = new short[1];
    this.stateList[0] = FODBCollectionDescriptor.TWO_WAY_SYNCHRO;
  }

  /**
   * if isCompressed return true, all stored data object in the collection are zipped.
   * To gzip collection allow to gain around 25% of the collection file length but increase the
   * synhro file size for complete synchro.
   * @param gzip
   */
  public boolean isCompressed() {
    return compressed;
  }

  /**
   * return  the way the synchronisation is done. 3 values are possible :
   * FODBCollectionDescriptor.TWO_WAY_SYNCHRO, FODBCollectionDescriptor.ONE_WAY_SEND_DATA, FODBCollectionDescriptor.ONE_WAY_GET_DATA
   * @return
   */
  public short getSynchroWay()	{
  	return this.stateList[0];
  }

  /**
   * set  the way the synchronisation is done. 3 values are possible :
   * FODBCollectionDescriptor.TWO_WAY_SYNCHRO, FODBCollectionDescriptor.ONE_WAY_SEND_DATA, FODBCollectionDescriptor.ONE_WAY_GET_DATA
   * @return
   */
  public void setSynchroWay(short way)	{
  	this.stateList[0] = way;
  }

  /**
   * if set to true, all stored data object in the colleciton are zipped.
   * To gzip collection allow to gain around 25% of the collection file length but increase the
   * synhro file size for complete synchro.
   * @param gzip
   */
  public void setCompressed(boolean compressed) {
    this.compressed = compressed;
  }

  public boolean isSynchronize() {
    return synchronize;
  }

  public void setSynchronize(boolean synchronize) {
    this.synchronize = synchronize;
  }

  public String getCollectionName() {
    return collectionName;
  }

  public Class getObjectType() {
    return objectType;
  }

	public String[] getSynchroDependList() {
		return synchroDependList;
	}

	public void setSynchroDependList(String[] synchroDependList) {
		this.synchroDependList = synchroDependList;
	}

}
